@@ -54,8 +54,8 @@
     _attrs = [[NSDictionary alloc] initWithObjectsAndKeys:[NSFont systemFontOfSize:8],NSFontAttributeName,nil];
     _cm = [ConfigManager globalConfig];
     [self setup];
-    [[NSNotificationCenter defaultCenter] addObserver:self 
-					     selector:@selector(configChanged:) 
+    [[NSNotificationCenter defaultCenter] addObserver:self
+					     selector:@selector(configChanged:)
 						 name:SAConfigManagerValueChanged object:_cm];
   }
   return self;
@@ -93,7 +93,7 @@
 
 NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 {
-  return [[[a valueForKey:@"object"] startDate] compare:[[b valueForKey:@"object"] startDate] withTime:YES];
+  return [[(Event *)[a valueForKey:@"object"] startDate] compare:[(Event *)[b valueForKey:@"object"] startDate] withTime:YES];
 }
 
 @implementation AppController
@@ -213,6 +213,7 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
     return self;
   [self initSummary];
   [[ConfigManager globalConfig] registerDefaults:[self defaults]];
+  _sm = [StoreManager globalManager];
   return self;
 }
 
@@ -239,43 +240,34 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 
   win = [NSApp iconWindow];
   width = [[win contentView] bounds].size.width;
-  height = [[win contentView] bounds].size.height;  
+  height = [[win contentView] bounds].size.height;
   [[win contentView] addSubview:AUTORELEASE([[AppIcon alloc] initWithFrame: NSMakeRect(1, 1, width - 2, height - 2)])];
 
   [self registerForServices];
   [NSApp setServicesProvider: self];
 
-  /* Set the selected day : this will update all views and titles (but not the summary) */
-  [calendar setDataSource:self];
-  /*
-   * FIXME : this has to be called before the StoreManager is setup
-   * so that the calendar date is setup when -dataChanged is called
-   * and the view redrawn by calling -reloadData
-   */
-  /*
-   * FIXME : setting the calendar date ends up calling -calendarView:selectedDateChanged
-   * then [DayView -setDate:] which calls [DayView -reloadData] where we find
-   * [StoreManager globalManager] => the StoreManager is hiddenly initialized
-   * 
-   * This is all too complex and should be redone maybe based on notifications
-   */
   [calendar setDate:[Date today]];
-  _sm = [StoreManager globalManager];
+  [calendar setDataSource:_sm];
+  [dayView setDataSource:_sm];
+  [weekView setDataSource:_sm];
 
-  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(dataChanged:) name:SADataChangedInStoreManager object:nil];
-  /* FIXME : this is overkill, we should only refresh the views for visual changes */
-  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(dataChanged:) name:SAStatusChangedForStore object:nil];
-
-  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reminderWillRun:) name:SAEventReminderWillRun object:nil];
   /* This will init the alarms for all loaded elements needing one */
   _am = [AlarmManager globalManager];
   _selm = [SelectionManager globalManager];
   _pc = [PreferencesController new];
+
+  [self updateSummaryData];
+
+  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(dataChanged:) name:SADataChangedInStoreManager object:nil];
+  /* FIXME : this is overkill, we should only refresh the views for visual changes */
+  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(dataChanged:) name:SAStatusChangedForStore object:nil];
+  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reminderWillRun:) name:SAEventReminderWillRun object:nil];
 }
 
 - (void)applicationWillTerminate:(NSNotification *)aNotification
 {
   [[NSNotificationCenter defaultCenter] removeObserver:self];
+  [_sm synchronise];
   RELEASE(_am);
   RELEASE(_sm);
   RELEASE(_summaryRoot);
@@ -319,7 +311,7 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   [_pc showPreferences];
 }
 
-NSComparisonResult compareEventTime(id a, id b, void *context)
+NSComparisonResult compareEventTime(Event  *a, Event *b, void *context)
 {
   return [[a startDate] compare:[b startDate] withTime:YES];
 }
@@ -327,7 +319,7 @@ NSComparisonResult compareEventTime(id a, id b, void *context)
 - (int)_sensibleStartForDuration:(int)duration
 {
   int minute = [dayView firstHour] * 60;
-  NSEnumerator *enumerator = [[[[_sm visibleAppointmentsForDay:_selectedDay] allObjects] 
+  NSEnumerator *enumerator = [[[[_sm visibleAppointmentsForDay:_selectedDay] allObjects]
 				sortedArrayUsingFunction:compareEventTime context:nil] objectEnumerator];
   Event *apt;
 
@@ -369,7 +361,7 @@ NSComparisonResult compareEventTime(id a, id b, void *context)
   NSArray *allTypes;
   Task *task;
 
-  allTypes = [pboard types];  
+  allTypes = [pboard types];
   if (![allTypes containsObject: NSStringPboardType]) {
     *error = @"No string type supplied on pasteboard";
     return;
@@ -474,7 +466,7 @@ NSComparisonResult compareEventTime(id a, id b, void *context)
       /* FIXME : this isn't enough : we have to find a writable store or error out */
       if (![store writable])
 	store = [_sm defaultStore];
-	
+
       start = [[el startDate] minuteOfDay];
       if ([_selm lastOperation] == SMCopy)
 	el = [el copy];
@@ -483,15 +475,15 @@ NSComparisonResult compareEventTime(id a, id b, void *context)
       if ([_selm lastOperation] == SMCopy) {
 	[store add:el];
 	/*
-	 * FIXME : the new event is now in store's dictionary, we 
-	 * should be able to release it. If we do, the application 
+	 * FIXME : the new event is now in store's dictionary, we
+	 * should be able to release it. If we do, the application
 	 * crashes when we delete this event, trying to release it
 	 * one time too many. I can't find the bug
 	 * [el release];
 	 */
       } else {
 	[store update:el];
-      }     
+      }
     }
     [date release];
   }
@@ -586,13 +578,10 @@ NSComparisonResult compareEventTime(id a, id b, void *context)
 
 - (void)dataChanged:(NSNotification *)not
 {
-  /* 
-   * FIXME : if a selected event was deleted by another application, 
+  /*
+   * FIXME : if a selected event was deleted by another application,
    * the selection will reference a non existing object
    */
-  [calendar reloadData];
-  [dayView reloadData];
-  [weekView reloadData];
   [taskView reloadData];
   [self performSearch];
   [self updateSummaryData];
@@ -676,7 +665,7 @@ NSComparisonResult compareEventTime(id a, id b, void *context)
 @end
 
 @implementation AppController(NSOutlineViewDataSource)
-- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
+- (NSInteger)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
 {
   if (item == nil)
     return [[_summaryRoot children] count];
@@ -688,7 +677,7 @@ NSComparisonResult compareEventTime(id a, id b, void *context)
     return YES;
   return [[item children] count] > 0;
 }
-- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item
+- (id)outlineView:(NSOutlineView *)outlineView child:(NSInteger)index ofItem:(id)item
 {
   if (item == nil)
     return [[_summaryRoot children] objectAtIndex:index];
@@ -713,15 +702,6 @@ NSComparisonResult compareEventTime(id a, id b, void *context)
 }
 @end
 
-@implementation AppController(CalendarViewDataSource)
-- (CVCellStatus)calendarView:(CalendarView *)view cellStatusForDate:(Date *)date
-{
-  if ([[_sm visibleAppointmentsForDay:date] count] > 0)
-    return CVHasDataCell;
-  return CVEmptyCell;
-}
-@end
-
 @implementation AppController(CalendarViewDelegate)
 - (void)calendarView:(CalendarView *)cs selectedDateChanged:(Date *)date
 {
@@ -766,8 +746,8 @@ NSComparisonResult compareEventTime(id a, id b, void *context)
   Date *date = [[calendar date] copy];
   [date setIsDate:NO];
   [date setMinute:start];
-  Event *apt = [[Event alloc] initWithStartDate:date 
-			               duration:end - start 
+  Event *apt = [[Event alloc] initWithStartDate:date
+			               duration:end - start
 			                  title:_(@"edit title...")];
   if (apt)
     [AppointmentEditor editorForEvent:apt];
@@ -784,19 +764,24 @@ NSComparisonResult compareEventTime(id a, id b, void *context)
 @end
 
 @implementation AppController(NSTableDataSource)
-- (int)numberOfRowsInTableView:(NSTableView *)aTableView
+- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView
 {
   return [[_sm visibleTasks] count];
 }
-- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
+- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex
 {
   Task *task = [[_sm visibleTasks] objectAtIndex:rowIndex];
+  Date *dueDate = [task dueDate];
 
   if ([[aTableColumn identifier] isEqualToString:@"summary"])
     return [task summary];
-  return [NSNumber numberWithInt:[task state]];
+  if ([[aTableColumn identifier] isEqualToString:@"state"])
+    return [NSNumber numberWithInt:[task state]];
+  if (dueDate != nil)
+     return [[dueDate calendarDate] descriptionWithCalendarFormat:[[NSUserDefaults standardUserDefaults] objectForKey:NSShortDateFormatString]];
+  return @"";
 }
-- (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
+- (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex
 {
   Task *task = [[_sm visibleTasks] objectAtIndex:rowIndex];
 
@@ -805,7 +790,7 @@ NSComparisonResult compareEventTime(id a, id b, void *context)
     [[task store] update:task];
   }
 }
-- (void)tableView:(NSTableView *)aTableView willDisplayCell:(id)aCell forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
+- (void)tableView:(NSTableView *)aTableView willDisplayCell:(id)aCell forTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex
 {
   Task *task;
 
