@@ -19,14 +19,29 @@
   NSMutableDictionary *_hreftree;
   NSMutableDictionary *_hrefresource;
   NSMutableArray *_modifiedhref;
+  NSString *_username;
+  NSString *_password;
 }
 @end
 
+@interface GroupDAVStore (WellKnownLookups)
+
+- (NSURL*)calendarURLFromRootOfServer:(NSURL *)originalURL
+				error:(NSError **)error;
+- (NSError *)unableToFindCalendarError:(NSString *)message;
+- (NSError *)reportGroupDAVError:(NSString *)title
+			 message:(NSString *)message;
+- (void)reportError:(NSError *)error;
+
+@end
+
 @interface GroupDAVDialog : NSObject
 {
   IBOutlet id panel;
   IBOutlet id name;
   IBOutlet id url;
+  IBOutlet id usernameField;
+  IBOutlet id passwordField;
   IBOutlet id cancel;
   IBOutlet id ok;
   IBOutlet id check;
@@ -35,6 +50,8 @@
 }
 - (BOOL)show;
 - (NSString *)url;
+- (NSString *)username;
+- (NSString *)password;
 - (NSString *)calendar;
 - (NSString *)task;
 - (void)selectItem:(id)sender;
@@ -55,6 +72,8 @@
       return nil;
     [name setStringValue:storeName];
     [url setStringValue:@"http://"];
+    [usernameField setStringValue:@""];
+    [passwordField setStringValue:@""];
     [self clearPopUps];
   }
   return self;
@@ -76,7 +95,7 @@
   else if (sender == ok)
     [NSApp stopModalWithCode:1];
   else if (sender == check)
-    [self controlTextDidEndEditing:nil];
+    [self checkDirectories];
 }
 
 - (void)updateOK
@@ -90,7 +109,176 @@
 {
   [self updateOK];
 }
-- (void)controlTextDidEndEditing:(NSNotification *)aNotification
+
+- (NSError *)unauthorizedAccessError
+{
+  return [self reportGroupDAVError: @"Unauthorized"
+			   message: @"Username and password are incorrect."];
+}
+
+- (NSError *)unableToFindCalendarError:(NSString *)message
+{
+  return [self reportGroupDAVError: @"Unable to find your calendar"
+			   message: message];
+}
+
+- (NSError *)reportGroupDAVError:(NSString *)title message:(NSString *)message
+{
+  id userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
+				title,  NSLocalizedDescriptionKey,
+			      message, NSLocalizedFailureReasonErrorKey,
+			      nil];
+
+  return [NSError errorWithDomain: @"GroupDAVStoreErrorDomain"
+			     code: 100
+			 userInfo: userInfo];
+}
+
+- (void)reportError:(NSError *)error
+{
+  NSRunAlertPanel([error localizedDescription],
+		  [error localizedFailureReason],
+		  @"OK",
+		  nil,
+		  nil);
+  [self updateOK];
+}
+
+- (NSURL*)calendarURLFromRootOfServer:(NSURL *)originalURL error:(NSError **)error
+{
+  id wellKnownURL = [originalURL URLByAppendingPathComponent:@".well-known/caldav"];
+  id resource = [[WebDAVResource alloc] initWithURL:wellKnownURL
+					   username:[usernameField stringValue]
+					   password:[passwordField stringValue]];
+  NSString *getUserPrincipalBody = @"<?xml version=\"1.0\" encoding=\"utf-8\"?><d:propfind xmlns:d=\"DAV:\"><d:prop><d:current-user-principal/></d:prop></d:propfind>";
+  NSString *getCalendarHomeSetBody = @"<?xml version=\"1.0\" encoding=\"utf-8\"?><d:propfind xmlns:d=\"DAV:\" xmlns:c=\"urn:ietf:params:xml:ns:caldav\"><d:self/><d:prop><c:calendar-home-set /></d:prop></d:propfind>";
+
+  [resource propfind:[getUserPrincipalBody dataUsingEncoding:NSUTF8StringEncoding] attributes:[NSDictionary dictionaryWithObject:@"Infinity" forKey:@"Depth"]];
+  if (([resource httpStatus] == 401) ||
+      ([resource httpStatus] == 403)) {
+    if (error) {
+      *error = [self unauthorizedAccessError];
+      [resource release];
+      return nil;
+    }
+  }
+  if ([resource data] == nil) {
+    if (error) {
+      *error = [self unableToFindCalendarError:
+		       [NSString stringWithFormat:
+				   @"No data was returned when "
+				 @"trying to find the current user "
+				 @"principal at %@",
+				 wellKnownURL]];
+    }
+    [resource release];
+    return nil;
+  };
+  id parser = [GSXMLParser parserWithData:[resource data]];
+  [resource release];
+  if ([parser parse]) {
+    id xpc = [[GSXPathContext alloc] initWithDocument:[[parser document] strippedDocument]];
+    GSXPathNodeSet *set = (GSXPathNodeSet *)[xpc evaluateExpression:@"(//response/propstat)[1]/prop/current-user-principal/href/text()"];
+    [xpc release];
+    if ([set count] == 0) {
+      if (error) {
+	*error = [self unableToFindCalendarError:
+			 [NSString stringWithFormat:
+				     @"Could not find current user "
+				   @"principal in data returned from "
+				   @"%@",
+				   wellKnownURL]];
+      }
+      return nil;
+    }
+    id node = [set nodeAtIndex:0];
+    id principalString = [node content];
+    id newURL = [originalURL URLByAppendingPathComponent:principalString];
+    if (newURL == nil) {
+      if (error) {
+	*error = [self unableToFindCalendarError:
+			 [NSString stringWithFormat:
+				     @"The current user "
+				   @"principal returned from "
+				   @"%@"
+				   @" is not a valid URL path.",
+				   wellKnownURL]];
+      }
+      return nil;
+    }
+    resource = [[WebDAVResource alloc] initWithURL:newURL
+					  username:[usernameField stringValue]
+					  password:[passwordField stringValue]];
+    [resource propfind:[getCalendarHomeSetBody dataUsingEncoding:NSUTF8StringEncoding] attributes:[NSDictionary dictionaryWithObject:@"Infinity" forKey:@"Depth"]];
+    if ([resource data] == nil) {
+      if (error) {
+	*error = [self unableToFindCalendarError:
+			 [NSString stringWithFormat:
+				     @"No data was returned when "
+				   @"trying to find your home  "
+				   @"calendar set at %@",
+				   newURL]];
+      }
+      [resource release];
+      return nil;
+    };
+    parser = [GSXMLParser parserWithData:[resource data]];
+    [resource release];
+    if ([parser parse]) {
+      xpc = [[GSXPathContext alloc] initWithDocument:[[parser document] strippedDocument]];
+      set = (GSXPathNodeSet *)[xpc evaluateExpression:@"(//response/propstat)[1]/prop/calendar-home-set/href/text()"];
+      [xpc release];
+      if ([set count] == 0) {
+	if (error) {
+	  *error = [self unableToFindCalendarError:
+			   [NSString stringWithFormat:
+				       @"Could not find home calendar "
+				     @"set in data returned from %@",
+				     wellKnownURL]];
+	}
+	return nil;
+      }
+      node = [set nodeAtIndex:0];
+      id homeSetString = [node content];
+      newURL = [originalURL URLByAppendingPathComponent:homeSetString];
+      if (newURL == nil) {
+	if (error) {
+	  *error = [self unableToFindCalendarError:
+			   [NSString stringWithFormat:
+				       @"Home calendar set"
+				     @"discovered via %@ "
+				     @"is not a valid URL path.",
+				     wellKnownURL]];
+	}
+	return nil;
+      }
+      return newURL;
+    }
+    else {
+      if (error) {
+	*error = [self unableToFindCalendarError:
+			 [NSString stringWithFormat:
+				     @"Couldn't parse the response to "
+				   @"a home calendar set request at "
+				   @"%@",
+				   newURL]];
+      }
+      return nil;
+    }
+  }
+  else {
+    if (error) {
+      *error = [self unableToFindCalendarError:
+		       [NSString stringWithFormat:
+				   @"Couldn't parse the response to "
+				 @"a get user principal request at %@",
+				 wellKnownURL]];
+    }
+    return nil;
+  }
+}
+
+- (void)checkDirectories
 {
   int i;
   WebDAVResource *resource;
@@ -98,35 +286,68 @@
   NSString *body = @"<?xml version=\"1.0\" encoding=\"utf-8\"?><propfind xmlns=\"DAV:\"><prop><getlastmodified/><executable/><resourcetype/></prop></propfind>";
   GSXPathContext *xpc;
   GSXPathNodeSet *set;
+  id originalURL = [NSURL URLWithString:[url stringValue]];
+  id newURL = nil;
 
-  [self clearPopUps];
-  if ([[url stringValue] isValidURL]) {
-    resource = [[WebDAVResource alloc] initWithURL:[NSURL URLWithString:[url stringValue]]];
-    if ([resource propfind:[body dataUsingEncoding:NSUTF8StringEncoding] attributes:[NSDictionary dictionaryWithObject:@"Infinity" forKey:@"Depth"]]) {
-      parser = [GSXMLParser parserWithData:[resource data]];
-      if ([parser parse]) {
-	xpc = [[GSXPathContext alloc] initWithDocument:[[parser document] strippedDocument]];
-	set = (GSXPathNodeSet *)[xpc evaluateExpression:@"//response[propstat/prop/resourcetype/vevent-collection]/href/text()"];
-	for (i = 0; i < [set count]; i++)
-	  [calendar addItemWithTitle:[[set nodeAtIndex:i] content]];
-	set = (GSXPathNodeSet *)[xpc evaluateExpression:@"//response[propstat/prop/resourcetype/vtodo-collection]/href/text()"];
-	for (i = 0; i < [set count]; i++)
-	  [task addItemWithTitle:[[set nodeAtIndex:i] content]];
-	RELEASE(xpc);
-	if ([calendar numberOfItems] > 0)
-	  [calendar selectItemAtIndex:1];
-	if ([task numberOfItems] > 0)
-	  [task selectItemAtIndex:1];
-      }
-    }
-    [resource release];
+  if (originalURL == nil) {
+    [self updateOK];
+    return;
   }
+  
+  [self clearPopUps];
+
+  if ([[originalURL path] length] == 0) {
+    NSError *error = nil;
+    newURL = [self calendarURLFromRootOfServer:originalURL error:&error];
+    if (newURL == nil) {
+      [self reportError:error];
+      return;
+    }
+  }
+  else {
+    newURL = originalURL;
+  }
+  resource = [[WebDAVResource alloc] initWithURL:newURL
+					username:[usernameField stringValue]
+					password:[passwordField stringValue]];
+  if ([resource propfind:[body dataUsingEncoding:NSUTF8StringEncoding] attributes:[NSDictionary dictionaryWithObject:@"Infinity" forKey:@"Depth"]]) {
+
+    parser = [GSXMLParser parserWithData:[resource data]];
+    if ([parser parse]) {
+      xpc = [[GSXPathContext alloc] initWithDocument:[[parser document] strippedDocument]];
+      set = (GSXPathNodeSet *)[xpc evaluateExpression:@"//response[propstat/prop/resourcetype/vevent-collection]/href/text()"];
+      for (i = 0; i < [set count]; i++)
+	[calendar addItemWithTitle:[[set nodeAtIndex:i] content]];
+      set = (GSXPathNodeSet *)[xpc evaluateExpression:@"//response[propstat/prop/resourcetype/vtodo-collection]/href/text()"];
+      for (i = 0; i < [set count]; i++)
+	[task addItemWithTitle:[[set nodeAtIndex:i] content]];
+      set = (GSXPathNodeSet *)[xpc evaluateExpression:@"//response[propstat/prop/resourcetype/calendar]/href/text()"];
+      for (i = 0; i < [set count]; i++) {
+	[calendar addItemWithTitle:[[set nodeAtIndex:i] content]];
+	[task addItemWithTitle:[[set nodeAtIndex:i] content]];
+      }
+      RELEASE(xpc);
+      if ([calendar numberOfItems] > 0)
+	[calendar selectItemAtIndex:1];
+      if ([task numberOfItems] > 0)
+	[task selectItemAtIndex:1];
+    }
+  }
+  [resource release];
   [self updateOK];
 }
 - (NSString *)url
 {
   return [url stringValue];
 }
+- (NSString *)username
+{
+  return [usernameField stringValue];
+}
+- (NSString *)password
+{
+  return [passwordField stringValue];
+}
 - (NSString *)calendar
 {
   if ([calendar indexOfItem:[calendar selectedItem]] == 0)
@@ -166,16 +387,25 @@
     _hrefresource = [[NSMutableDictionary alloc] initWithCapacity:512];
     _modifiedhref = [NSMutableArray new];
     _url = [[NSURL alloc] initWithString:[[self config] objectForKey:ST_URL]];
+    _username = [[[self config] objectForKey:ST_USERNAME] copy];
+    _password = [[[self config] objectForKey:ST_PASSWORD] copy];
+    if (_username == nil) {
+      _username = [[_url user] copy];
+    }
+    if (_password == nil) {
+      _password = [[_url password] copy];
+    }
+    
     _calendar = nil;
     _task = nil;
     if ([[self config] objectForKey:ST_CALENDAR_URL])
-      _calendar = [[WebDAVResource alloc] initWithURL:[[NSURL alloc] initWithString:[[self config] objectForKey:ST_CALENDAR_URL]] authFromURL:_url];
+      _calendar = [[WebDAVResource alloc] initWithURL:[[NSURL alloc] initWithString:[[self config] objectForKey:ST_CALENDAR_URL]] username: _username password: _password];
     if ([[self config] objectForKey:ST_TASK_URL])
-      _task = [[WebDAVResource alloc] initWithURL:[[NSURL alloc] initWithString:[[self config] objectForKey:ST_TASK_URL]] authFromURL:_url];
+      _task = [[WebDAVResource alloc] initWithURL:[[NSURL alloc] initWithString:[[self config] objectForKey:ST_TASK_URL]] username: _username password: _password];
     [self read];
-    [[NSNotificationCenter defaultCenter] addObserver:self 
-					     selector:@selector(configChanged:) 
-						 name:SAConfigManagerValueChanged 
+    [[NSNotificationCenter defaultCenter] addObserver:self
+					     selector:@selector(configChanged:)
+						 name:SAConfigManagerValueChanged
 					       object:[self config]];
   }
   return self;
@@ -201,9 +431,10 @@
       calendarURL = [NSURL URLWithString:[dialog calendar] possiblyRelativeToURL:baseURL];
     if ([dialog task])
       taskURL = [NSURL URLWithString:[dialog task] possiblyRelativeToURL:baseURL];
-    [dialog release];
     cm = [[ConfigManager alloc] initForKey:name];
     [cm setObject:[dialog url] forKey:ST_URL];
+    [cm setObject:[dialog username] forKey:ST_USERNAME];
+    [cm setObject:[dialog password] forKey:ST_PASSWORD];
     if (calendarURL)
       [cm setObject:[calendarURL absoluteString] forKey:ST_CALENDAR_URL];
     if (taskURL)
@@ -231,6 +462,8 @@
   [_hreftree release];
   [_hrefresource release];
   [_modifiedhref release];
+  [_username release];
+  [_password release];
   [super dealloc];
 }
 
@@ -249,7 +482,9 @@
     url = [NSURL URLWithString:[NSString stringWithFormat:@"%@/%@", [[_calendar url] absoluteString], [elt UID]]];
   else
     url = [NSURL URLWithString:[NSString stringWithFormat:@"%@/%@", [[_task url] absoluteString], [elt UID]]];
-  resource = [[WebDAVResource alloc] initWithURL:url authFromURL:_url];
+  resource = [[WebDAVResource alloc] initWithURL:url
+					username:_username
+					password:_password];
   tree = [iCalTree new];
   if ([tree add:elt]) {
     [resource put:[tree iCalTreeAsData] attributes:[NSDictionary dictionaryWithObjectsAndKeys:@"text/calendar; charset=utf-8", @"Content-Type", @"*", @"If-None-Match", nil, nil]];
@@ -297,8 +532,8 @@
 - (void)read
 {
   if ([self enabled])
-    [[[StoreManager globalManager] operationQueue] addOperation:[[[InvocationOperation alloc] initWithTarget:self 
-												    selector:@selector(doRead) 
+    [[[StoreManager globalManager] operationQueue] addOperation:[[[InvocationOperation alloc] initWithTarget:self
+												    selector:@selector(doRead)
 												      object:nil] autorelease]];
 }
 
@@ -306,7 +541,7 @@
 {
   if (![self enabled] || ![self writable])
     return;
-  [[[StoreManager globalManager] operationQueue] addOperation:[[[InvocationOperation alloc] initWithTarget:self 
+  [[[StoreManager globalManager] operationQueue] addOperation:[[[InvocationOperation alloc] initWithTarget:self
 												  selector:@selector(doWrite)
 												    object:nil] autorelease]];
 }
@@ -334,7 +569,7 @@ static NSString * const EXPRGETHREF = @"//response[propstat/prop/getetag]/href/t
   GSXPathNodeSet *set;
   NSURL *elementURL;
 
-  if (![ressource propfind:[PROPFINDGETETAG dataUsingEncoding:NSUTF8StringEncoding] 
+  if (![ressource propfind:[PROPFINDGETETAG dataUsingEncoding:NSUTF8StringEncoding]
 		attributes:[NSDictionary dictionaryWithObject:@"1" forKey:@"Depth"]])
     return nil;
   result = [NSMutableArray arrayWithCapacity:256];
@@ -362,7 +597,9 @@ static NSString * const EXPRGETHREF = @"//response[propstat/prop/getetag]/href/t
 
   enumerator = [items objectEnumerator];
   while ((href = [enumerator nextObject])) {
-    element = [[WebDAVResource alloc] initWithURL:href authFromURL:_url];
+    element = [[WebDAVResource alloc] initWithURL:href
+					 username:_username
+					 password:_password];
     tree = [iCalTree new];
     if ([element get] && [tree parseData:[element data]]) {
       components = [tree components];
@@ -399,13 +636,13 @@ static NSString * const EXPRGETHREF = @"//response[propstat/prop/getetag]/href/t
   }
   if (error) {
     NSLog(@"Error while reading %@", [self description]);
-    [[NSNotificationCenter defaultCenter] postNotificationName:SAErrorReadingStore 
+    [[NSNotificationCenter defaultCenter] postNotificationName:SAErrorReadingStore
 							object:self
 						      userInfo:[NSDictionary dictionary]];
   } else {
     [self performSelectorOnMainThread:@selector(fillWithElements:) withObject:loadedData waitUntilDone:YES];
-    NSLog(@"GroupDAVStore from %@ : loaded %d appointment(s)", [_url anonymousAbsoluteString], [[self events] count]);
-    NSLog(@"GroupDAVStore from %@ : loaded %d tasks(s)", [_url anonymousAbsoluteString], [[self tasks] count]);
+    NSLog(@"GroupDAVStore from %@ : loaded %lu appointment(s)", [_url anonymousAbsoluteString], [[self events] count]);
+    NSLog(@"GroupDAVStore from %@ : loaded %lu tasks(s)", [_url anonymousAbsoluteString], [[self tasks] count]);
   }
   [loadedData release];
 }
@@ -426,7 +663,7 @@ static NSString * const EXPRGETHREF = @"//response[propstat/prop/getetag]/href/t
     element = [_hrefresource objectForKey:href];
     tree = [_hreftree objectForKey:href];
     if ([element put:[tree iCalTreeAsData] attributes:attr]) {
-      /* Read it back to update the attributes */ 
+      /* Read it back to update the attributes */
       /* FIXME : RFC says we should update the list instead */
       [element updateAttributes];
       [_modifiedhref removeObject:href];
@@ -438,7 +675,7 @@ static NSString * const EXPRGETHREF = @"//response[propstat/prop/getetag]/href/t
   }
   if (error) {
     NSLog(@"Calendar %@ will be made read only and data reread", [self description]);
-    [[NSNotificationCenter defaultCenter] postNotificationName:SAErrorWritingStore 
+    [[NSNotificationCenter defaultCenter] postNotificationName:SAErrorWritingStore
 							object:self
 						      userInfo:[NSDictionary dictionary]];
   }
